import { addBreadcrumb, captureException } from '@sentry/minimal';

const CATEGORY = 'relay';
const isExecuteEvent = (logEvent) => logEvent.name.startsWith('execute');
const isQueryResourceEvent = (logEvent) => logEvent.name.startsWith('queryresource');
const errorsIsGraphQLError = (errors) => Array.isArray(errors) && errors.some((item) => 'message' in item);
const logFunction = ({ tag = 'data.client', } = {}) => (logEvent) => {
    var _a, _b;
    const category = `${CATEGORY}.${logEvent.name}`;
    if (isExecuteEvent(logEvent)) {
        const { transactionID } = logEvent;
        switch (logEvent.name) {
            case 'execute.start': {
                const params = logEvent.params;
                addBreadcrumb({
                    type: 'info',
                    category,
                    data: {
                        transactionID,
                        id: (_a = params.id) !== null && _a !== void 0 ? _a : params.cacheID,
                        kind: params.operationKind,
                        name: params.name,
                        variables: logEvent.variables,
                    },
                });
                break;
            }
            case 'execute.error': {
                let errors = 'na';
                const error = logEvent.error;
                if ('graphqlErrors' in error &&
                    errorsIsGraphQLError(error === null || error === void 0 ? void 0 : error.graphqlErrors)) {
                    errors = error.graphqlErrors;
                    delete error.graphqlErrors;
                }
                captureException(error, {
                    tags: {
                        [tag]: 'relay',
                    },
                    contexts: {
                        relay: {
                            transactionID,
                            name: logEvent.name,
                            errors,
                        },
                    },
                });
                break;
            }
            default: {
                addBreadcrumb({
                    type: 'debug',
                    category,
                    data: {
                        transactionID,
                    },
                });
                break;
            }
        }
    }
    else if (isQueryResourceEvent(logEvent)) {
        const { resourceID } = logEvent;
        switch (logEvent.name) {
            case 'queryresource.fetch': {
                const params = logEvent.operation.request.node.params;
                addBreadcrumb({
                    type: 'info',
                    category,
                    data: {
                        resourceID,
                        id: (_b = params.id) !== null && _b !== void 0 ? _b : params.cacheID,
                        kind: params.operationKind,
                        name: params.name,
                        variables: logEvent.operation.root.variables,
                        shouldFetch: logEvent.shouldFetch ? 'yes' : 'no',
                        fetchPolicy: logEvent.fetchPolicy,
                        renderPolicy: logEvent.renderPolicy,
                        queryAvailability: logEvent.queryAvailability,
                    },
                });
                break;
            }
            default: {
                addBreadcrumb({
                    type: 'debug',
                    category,
                    data: {
                        resourceID,
                    },
                });
                break;
            }
        }
    }
    else {
        addBreadcrumb({
            type: 'debug',
            category,
        });
    }
};

export { logFunction };
